;+
;   
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU Lesser General Public License as published
;    by the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU Lesser General Public License for more details.
;
;    You should have received a copy of the GNU Lesser General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
;-

;+
; NAME:        
;       HW_ALPHA
; PURPOSE:              
;       GUI for the OMEGA science team SOFTXX software
; EXPLANATION:               
;       The SOFTXX software of the OMEGA science team is designed
;       for batch processing under UNIX/Linux. This IDL script adds
;       a simple GUI for interactive processing.
;               
; CALLING SEQUENCE: 
;       HW_ALPHA
;    
; INPUT PARAMETERS:     
;       None, all input is interactively acquired.
;
; OUTPUTS:
;       5 ENVI images (with headers):
;       <filename>_mola     - MOLA altitude data
;       <filename>_lonlat   - LONLAT data for building a GLT
;       <filename>_idat     - OMEGA raw data 
;       <filename>_jdat     - OMEGA radiance data 
;       <filename>_geocube  - Housekeeping data   
;       1 ASCII file:
;       <filename>_specmars - Solar spectrum at data acquisition 
;                              
; OPTIONAL INPUT KEYWORD:
;       None.
;
; PROCEDURE CALLS:
;       None.
;
; MODIFICATION HISTORY: 
;       1.0 Written,       hvdw, Spring 2009
;       1.1 Modified,      hvdw, Januari 2011
;                          - adjusted to SOFT07,
;       1.2 Modified,      hvdw, Februari 2011
;                          - added user config,
;                          - added license, help
;       09  Modified       hvdw, November 2013
;                          - adjusted to SOFT09
;       10  Modified       hvdw, June 2016
;                          - adjusted to SOFT10
;- 

;+
; v.20131101a
;-

pro hw_alpha_event, ev

  compile_opt idl2

  catch, theerror
  if theerror ne 0 then begin
    catch, /cancel
    void = dialog_message(!error_state.msg, /error,/center)
    return
  endif

  widget_control, ev.id,  get_uvalue = id
  widget_control, ev.top, get_uvalue = info

  case id of
    'QUB' : begin
              widget_control, info.qubpath_id, get_value=oldpath
              newpath = dialog_pickfile( $
                dialog_parent = ev.top, $
                title = 'Please select an OMEGA .QUB file', $
                filter = [ '*.QUB','*.qub' ], $
                path = file_dirname(oldpath,/mark_directory), $
                /multiple_files, $
                /read, $
                /must_exist )
              if newpath[0] ne '' then begin
                widget_control, info.qubpath_id, set_value = newpath
                widget_control, info.navpath_id, set_value = $
                  file_dirname(newpath[0])+'/'+file_basename(newpath[0],'.QUB')+'.NAV'
              endif     
            end
    'NAV' : begin
              widget_control, info.navpath_id, get_value=oldpath
              newpath = dialog_pickfile( $
                dialog_parent = ev.top, $
                title = 'Please select a corresponding .NAV file', $
                filter = [ '*.NAV','*.nav' ], $
                path = file_dirname(oldpath), $
                /multiple_files, $
                /read, $
                /must_exist )
              if newpath[0] ne '' then $
                widget_control, info.navpath_id, set_value = newpath
              end
    'OMG' : begin
              widget_control, info.omgpath_id, get_value = oldpath
              newpath = dialog_pickfile( $
                dialog_parent = ev.top, $
                title = 'Please select the SOFT10 directory', $
                path = oldpath,$
                /directory,$
                /must_exist )  
              if newpath ne '' then $
                widget_control, info.omgpath_id, set_value = newpath
              end
    'SAV' : begin
              widget_control, info.savpath_id, get_value=oldpath
              newpath = dialog_pickfile( $
                dialog_parent = ev.top, $
                title = 'Please select an output directory',$
                path = oldpath,$
                /directory,$
                /write,$
                /must_exist )
              if newpath ne '' then $
                widget_control, info.savpath_id, set_value = newpath
              end
    'OPTIONS': widget_control, ev.top, set_uvalue = info
    'RUN'    : hw_alpha_doit, ev 
    'HELP'   : hw_alpha_help, ev
    'DEBUG'  : hw_alpha_debug,ev
    'LICENSE': hw_alpha_license,ev
    'ABOUT'  : hw_alpha_about,ev
    'SAVDONE': begin
                 hw_alpha_write_config,ev
                 widget_control, ev.top, /destroy
                 return
               end
    'DONE'   : begin    
                 widget_control, ev.top, /destroy
                 return
               end      
    else     : print, 'DEBUG: unknown event.id '+string(id)
    
  endcase
 
  ; enable run button if OMEGA files,
  ; SOFTXX and output paths are defined
  widget_control, info.qubpath_id, get_value = qubpath
  widget_control, info.navpath_id, get_value = navpath
  widget_control, info.omgpath_id, get_value = omgpath
  widget_control, info.savpath_id, get_value = savpath
  
  flag = 1
  if qubpath eq '' then flag = 0
  if navpath eq '' then flag = 0
  if omgpath eq '' then flag = 0
  if savpath eq '' then flag = 0
  if flag eq 1 $
    then widget_control, info.run_id, sensitive=1 $
    else widget_control, info.run_id, sensitive=0
    
  widget_control, ev.top, set_uvalue = info

end

;+
; hw_alpha, main program, state definition and GUI
;-

pro hw_alpha

  compile_opt idl2
  
  ; catch errors and display in window
  catch, theerror
  if theerror ne 0 then begin
    catch, /cancel
    void = dialog_message(!error_state.msg,/error,/center)
    return
  endif

  ; suppress error messages to console
  !except = 0
  
  ; get program configuration
  config = hw_alpha_read_config( hw_alpha_config_dir() )

  ; info structure with program settings & status
  info = { $
    program   : 'AlphA', $
    version   : '10', $
    author    : 'Harald van der Werff', $
    copyright : 'University of Twente', $
    copyyear  : '2009 - 2017',$
    cnfpath   : config.cnfpath, $
    qubpath_id: 0l, $
    navpath_id: 0l, $
    omgpath_id: 0l, $
    savpath_id: 0l, $
    options_id: 0l, $
    run_id    : 0l  $
  }

  base = widget_base(title=info.program+' '+info.version,mbar=mbar,/column )
  
  m1 = widget_button(mbar,value='Program',/menu)
    void = widget_button($
      m1,$
      value='Save and Exit ',$
      uvalue='SAVDONE',$
      /separator)
    void = widget_button($
      m1,$
      value='Exit',$
      uvalue='DONE')
  m2 = widget_button($
    mbar,$
    value='Help',$
    /menu,$
    /help)
    void = widget_button($
      m2,$
      value='Help',$
      uvalue='HELP')
    void = widget_button($
      m2,$
      value='Debug',$
      uvalue='DEBUG',$
      /separator)
    void = widget_button($
      m2,$
      value='License',$
      uvalue='LICENSE',$
      /separator)
    void = widget_button($
      m2,$
      value='About',$
      uvalue='ABOUT',$
      /separator)

   base2 = widget_base(base, row=3, /frame)
     void = widget_label(base2,value='Input selection',/align_left)
     void = widget_label(base2,value='')
     void = widget_button(base2, value = 'Select OMEGA .QUB', $
       uvalue = 'QUB', xsize=130)
     info.qubpath_id = widget_text(base2, xsize=50, $
       value = file_dirname(config.qubpath,/mark_directory))
     void = widget_button(base2, value = 'Select OMEGA .NAV', $
       uvalue = 'NAV', xsize=130)
     info.navpath_id = widget_text(base2, xsize=50, $
       value = file_dirname(config.navpath,/mark_directory))
   
   base3 = widget_base(base, row=3, /frame)
     void = widget_label(base3,value='Directory selection',/align_left)
     void = widget_label(base3,value='')
     void = widget_button(base3, value = 'Select SOFTXX dir', $
       uvalue = 'OMG', xsize=130)
     info.omgpath_id = widget_text(base3, xsize=50, $
       value = config.omgpath)
     void = widget_button(base3, value = 'Select Output dir', $
       uvalue = 'SAV', xsize=130)
     info.savpath_id = widget_text(base3, xsize=50, $
       value = config.savpath)

   base4 = widget_base(base,/column,/frame)
     void = widget_label($
       base4,$
       value='Requested output',$
       /align_left)
     values = [ $
       'idat     - OMEGA - raw data (16-bit integer)',$
       'jdat     - OMEGA - radiance data (32-bit float)',$
       'alt      - MOLA  - alitude data (32-bit float)',$
       'lonlat   - OMEGA - geographic coordinates C pixels (32-bit float)',$
       'geocube  - OMEGA - housekeeping data (32-bit float)',$
       'specmars - OMEGA - wavelength and solar spectrum (ASCII)' ]
     info.options_id = cw_bgroup($
       base4,$
       values,$
       set_value = config.options,$
       /column,$
       /nonexclusive,$
       uvalue = 'OPTIONS')
     info.run_id = widget_button($
       base,$
       value = 'Run SOFTXX and Save to disk', $
       uvalue = 'RUN',$
       sensitive = 0,$
       xsize = 80)
   
    base5 = widget_base($
      base,$
      /row,$
      /frame)
      void = widget_label($
        base5,$
        value = info.author+', '+info.copyright+' ('+info.copyyear+')')

  widget_control,base,/realize,set_uvalue = info

  xmanager,'hw_alpha',base,/no_block

end 
